<?php

namespace InSegment\WorkersFramework\Console;

use Shavshukov\EventAppBase\Application as BaseApplication;
use Shavshukov\EventAppBase\Interfaces\ModelInterface;

use InSegment\WorkersFramework\Models\Config;

class Application extends BaseApplication
{
    /**
     * Filename of config file or a model to source from
     *
     * @var string|\Shavshukov\EventAppBase\Interfaces\ModelInterface
     */
    protected $configSource;
    
    /**
     * Constructor
     * 
     * @param string $appPath
     * @param string|\Shavshukov\EventAppBase\Interfaces\ModelInterface $configSource
     * @see \Shavshukov\EventAppBase\Interfaces\ModelInterface extends StorageInterface, ValidatableInterface
     */
    public function __construct(string $appPath, $configSource = Config::DEFAULT_CONFIG_FILE)
    {
        $this->configSource = $configSource;
                
        parent::__construct($appPath);
    }
    
    /**
     * Handle console command and return a code
     * 
     * @return int
     */
    public function handle()
    {
        $kernel = new Kernel($this);
        
        return $kernel->handle();
    }

    /**
     * {@inheritDoc}
     */
    public function description(): string
    {
        return 'InSegment Queue Workers Framework';
    }

    /**
     * {@inheritDoc}
     */
    protected function getConfigInstance(): ModelInterface
    {
        return $this->getConfigReader()->readConfigFromConfigSource($this, $this->configSource);
    }
    
    /**
     * @return \InSegment\WorkersFramework\Console\ReadConfig
     */
    protected function getConfigReader(): ReadConfig
    {
        return new ReadConfig;
    }
    
}
