<?php

namespace InSegment\WorkersFramework\Services;

use Shavshukov\EventAppBase\Model\JsonModel;

use InSegment\WorkersFramework\Interfaces\QueueMessageInterface;
use InSegment\WorkersFramework\Interfaces\QueueInterface;
use InSegment\WorkersFramework\Models\State;

class StateConsumer extends AbstractConsumer
{
    
    /**
     * State instance
     *
     * @var \InSegment\WorkersFramework\Models\State
     */
    protected $state;
    
    /**
     * Constructor
     * 
     * @param \InSegment\WorkersFramework\Services\QueueInterface $queue
     * @param \InSegment\WorkersFramework\Models\State $state
     */
    public function __construct(QueueInterface $queue, State $state)
    {
        parent::__construct($queue);
        $this->state = $state;
    }
    
    /**
     * Move message from the main queue to the worker queue and acknowledge the original one
     * 
     * @param \InSegment\WorkersFramework\Interfaces\QueueMessageInterface $message
     * @return mixed
     */
    public function receiveMessage(QueueMessageInterface $message)
    {
        $updates = JsonModel::from($message->getContents());
        $microtimeUpdate = (float) $updates->get('timestamp');
        $microtimeCurrentState = (float) $this->state->get('timestamp');
        if ($microtimeUpdate > $microtimeCurrentState) {
            $this->trigger('updates', $updates);
        }
        
        return $message->acknowlege();
    }

}
